import logging
import os
import re
from py2neo import Graph, Node, Relationship
from typing import Set, List, Dict, Tuple

# 配置日志
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

class KnowledgeGraph:
    def __init__(self, triples: List[Dict], document_id: str):
        self.triples = triples
        self.document_id = document_id
        self.g = Graph("bolt://10.18.34.18:7687", auth=('neo4j', '261229..wmh'))
        logger.info(f"开始构建知识图谱，document_id: {self.document_id}, 三元组数量: {len(triples)}")

    def _normalize_label(self, entity_type: str) -> str:
        """规范化实体类型为 Neo4j 标签"""
        if not entity_type or not entity_type.strip():
            return 'Entity'
        entity_type = re.sub(r'[^\w\s]', '', entity_type.strip())
        words = entity_type.split()
        label = '_'.join(word.capitalize() for word in words if word)
        return label or 'Entity'

    def _clean_relation(self, relation: str) -> Tuple[str, str]:
        """清洗关系，返回 (rel_type, rel_name)"""
        relation = relation.strip()
        if not relation:
            return 'RELATED_TO', '相关'
        if relation.startswith('<') and relation.endswith('>'):
            cleaned_relation = relation[1:-1]
            rel_name = cleaned_relation
            rel_type = re.sub(r'[^\w\s]', '', cleaned_relation).replace(' ', '_').upper()
        else:
            rel_name = relation
            rel_type = re.sub(r'[^\w\s]', '', relation).replace(' ', '_').upper()
            if 'instance of' in relation.lower():
                rel_type = 'INSTANCE_OF'
                rel_name = '实例'
            elif 'subclass of' in relation.lower():
                rel_type = 'SUBCLASS_OF'
                rel_name = '子类'
            elif 'part of' in relation.lower():
                rel_type = 'PART_OF'
                rel_name = '部分'
        logger.debug(f"处理关系: {relation} -> {rel_type} ({rel_name})")
        return rel_type, rel_name

    def read_nodes(self) -> Tuple[Dict[str, Set], Dict[str, List], List[Dict]]:
        """从三元组列表中读取节点和关系"""
        nodes_by_label = {}
        relations_by_type = {}
        triples = []

        try:
            for triple in self.triples:
                if not all(key in triple for key in ['head', 'head_type', 'type', 'tail', 'tail_type']):
                    logger.warning(f"无效三元组: {triple}")
                    continue
                head, relation, tail, head_type, tail_type = (
                    triple['head'], triple['type'], triple['tail'], triple['head_type'], triple['tail_type']
                )
                head_label = self._normalize_label(head_type)
                tail_label = self._normalize_label(tail_type)
                logger.debug(f"实体类型: {head_type} -> {head_label}, {tail_type} -> {tail_label}")

                if head_label not in nodes_by_label:
                    nodes_by_label[head_label] = set()
                if tail_label not in nodes_by_label:
                    nodes_by_label[tail_label] = set()
                nodes_by_label[head_label].add(head)
                nodes_by_label[tail_label].add(tail)

                rel_type, rel_name = self._clean_relation(relation)
                if rel_type not in relations_by_type:
                    relations_by_type[rel_type] = []
                relations_by_type[rel_type].append({
                    'head': head,
                    'tail': tail,
                    'head_label': head_label,
                    'tail_label': tail_label,
                    'rel_name': rel_name
                })

                triples.append({
                    'head': head,
                    'relation': relation,
                    'tail': tail,
                    'head_type': head_type,
                    'tail_type': tail_type
                })

            logger.info(f"读取节点: {sum(len(nodes) for nodes in nodes_by_label.values())} 个")
            logger.info(f"读取关系: {sum(len(rels) for rels in relations_by_type.values())} 条")
            return nodes_by_label, relations_by_type, triples

        except Exception as e:
            logger.error(f"读取三元组失败: {str(e)}")
            raise RuntimeError(f"读取三元组失败: {str(e)}")

    def create_node(self, label: str, nodes: Set[str]):
        """创建节点，包含 document_id 属性"""
        count = 0
        for node_name in nodes:
            query = f"MATCH (n:{label} {{name: '{node_name}', document_id: '{self.document_id}'}}) RETURN n"
            try:
                if self.g.run(query).data():
                    continue
                node = Node(label, name=node_name, document_id=self.document_id)
                self.g.create(node)
                count += 1
                logger.debug(f"创建节点: {label} - {node_name} (document_id: {self.document_id})")
            except Exception as e:
                logger.error(f"创建节点失败: {label} - {node_name}, 错误: {str(e)}")
        logger.info(f"创建 {label} 节点: {count}/{len(nodes)} 个")
        return count

    def create_relationship(self, rel_type: str, relations: List[Dict]):
        """创建关系"""
        count = 0
        total = len(relations)
        seen_edges = set()
        for rel in relations:
            head, tail, head_label, tail_label, rel_name = (
                rel['head'], rel['tail'], rel['head_label'], rel['tail_label'], rel['rel_name']
            )
            edge_key = f"{head_label}:{head}###{tail_label}:{tail}###{rel_type}"
            if edge_key in seen_edges:
                continue
            seen_edges.add(edge_key)

            query = (
                f"MATCH (p:{head_label} {{name: '{head}', document_id: '{self.document_id}'}}), "
                f"(q:{tail_label} {{name: '{tail}', document_id: '{self.document_id}'}}) "
                f"CREATE (p)-[r:{rel_type} {{name: '{rel_name}'}}]->(q)"
            )
            try:
                self.g.run(query)
                count += 1
                logger.debug(f"创建关系: {head} -[{rel_type}]-> {tail} (document_id: {self.document_id})")
            except Exception as e:
                logger.error(f"创建关系失败: {query}, 错误: {str(e)}")
        logger.info(f"创建 {rel_type} 关系: {count}/{total} 条")
        return count

    def create_graphnodes(self):
        """创建所有节点"""
        nodes_by_label, _, _ = self.read_nodes()
        total = 0
        for label, nodes in nodes_by_label.items():
            total += self.create_node(label, nodes)
        logger.info(f"总计创建节点: {total} 个")
        return total

    def create_graphrels(self):
        """创建所有关系"""
        _, relations_by_type, _ = self.read_nodes()
        total = 0
        for rel_type, relations in relations_by_type.items():
            total += self.create_relationship(rel_type, relations)
        logger.info(f"总计创建关系: {total} 条")
        return total

    def export_data(self):
        """导出节点到文件，包含 document_id"""
        nodes_by_label, _, _ = self.read_nodes()
        os.makedirs('dict', exist_ok=True)
        for label, nodes in nodes_by_label.items():
            with open(f'dict/{label.lower()}.txt', 'w', encoding='utf-8') as f:
                f.write('\n'.join(f"{name}\t{self.document_id}" for name in sorted(nodes)))
            logger.info(f"导出 {label} 节点到 dict/{label.lower()}.txt: {len(nodes)} 个")
        return