import os
import logging
from typing import Dict
from pymilvus import connections, utility, Collection, CollectionSchema, FieldSchema, DataType
from llmengine.base_collection import BaseCollection, collection_register

logger = logging.getLogger(__name__)

CONFIG_PATH = os.getenv('CONFIG_PATH', '/share/wangmeihua/rag/conf/milvusconfig.yaml')
try:
    with open(CONFIG_PATH, 'r', encoding='utf-8') as f:
        import yaml
        config = yaml.safe_load(f)
    MILVUS_DB_PATH = config['database']['milvus_db_path']
except Exception as e:
    logger.error(f"加载配置文件 {CONFIG_PATH} 失败: {str(e)}")
    raise RuntimeError(f"无法加载配置文件: {str(e)}")

class MilvusCollection(BaseCollection):
    def __init__(self, model_id):
        self.model_id = model_id
        self.db_path = MILVUS_DB_PATH

    def ensure_milvus_directory(self) -> None:
        """确保 Milvus 数据库目录存在"""
        db_dir = os.path.dirname(self.db_path)
        if not os.path.exists(db_dir):
            os.makedirs(db_dir, exist_ok=True)
            logger.debug(f"创建 Milvus 目录: {db_dir}")
        if not os.access(db_dir, os.W_OK):
            raise RuntimeError(f"Milvus 目录 {db_dir} 不可写")

    def initialize_milvus_connection(self) -> None:
        """初始化 Milvus 连接"""
        try:
            if not connections.has_connection("default"):
                connections.connect("default", uri=self.db_path)
                logger.debug(f"已连接到 Milvus Lite，路径: {self.db_path}")
            else:
                logger.debug("已存在 Milvus 连接，跳过重复连接")
        except Exception as e:
            logger.error(f"连接 Milvus 失败: {str(e)}")
            raise RuntimeError(f"连接 Milvus 失败: {str(e)}")

    def cleanup_milvus_connection(self) -> None:
        """清理 Milvus 连接"""
        try:
            if connections.has_connection("default"):
                connections.disconnect("default")
                logger.debug("已断开 Milvus 连接")
        except Exception as e:
            logger.warning(f"断开 Milvus 连接失败: {str(e)}")

    def create_collection(self, db_type: str) -> Dict:
        """
        为指定 db_type 创建 Milvus 集合。
        返回创建结果，包含集合名称和状态。
        """
        try:
            # 参数验证
            if not db_type:
                raise ValueError("db_type 不能为空")
            if "_" in db_type:
                raise ValueError("db_type 不能包含下划线")
            if len(db_type) > 100:
                raise ValueError("db_type 的长度应小于 100")

            self.ensure_milvus_directory()
            self.initialize_milvus_connection()

            # 集合名称
            collection_name = f"ragdb_{db_type}"
            if len(collection_name) > 255:
                raise ValueError(f"集合名称 {collection_name} 超过 255 个字符")
            logger.debug(f"集合名称: {collection_name}")

            # 如果集合已存在，检查 schema 兼容性
            if utility.has_collection(collection_name):
                try:
                    collection = Collection(collection_name)
                    collection.load()
                    logger.debug(f"集合 {collection_name} 已存在并加载成功")
                    return {
                        "status": "success",
                        "collection_name": collection_name,
                        "message": f"集合 {collection_name} 已存在"
                    }
                except Exception as e:
                    logger.error(f"加载集合 {collection_name} 失败: {str(e)}")
                    raise RuntimeError(f"加载集合失败: {str(e)}")

            # 定义 schema
            fields = [
                FieldSchema(name="pk", dtype=DataType.VARCHAR, is_primary=True, max_length=36, auto_id=True),
                FieldSchema(name="userid", dtype=DataType.VARCHAR, max_length=100),
                FieldSchema(name="document_id", dtype=DataType.VARCHAR, max_length=36),
                FieldSchema(name="text", dtype=DataType.VARCHAR, max_length=65535),
                FieldSchema(name="vector", dtype=DataType.FLOAT_VECTOR, dim=1024),
                FieldSchema(name="filename", dtype=DataType.VARCHAR, max_length=255),
                FieldSchema(name="file_path", dtype=DataType.VARCHAR, max_length=1024),
                FieldSchema(name="upload_time", dtype=DataType.VARCHAR, max_length=64),
                FieldSchema(name="file_type", dtype=DataType.VARCHAR, max_length=64),
            ]
            schema = CollectionSchema(
                fields=fields,
                description=f"{db_type} 数据集合，跨用户使用，包含 document_id 和元数据字段",
                auto_id=True,
                primary_field="pk",
            )

            # 创建集合
            try:
                collection = Collection(collection_name, schema)
                collection.create_index(
                    field_name="vector",
                    index_params={"index_type": "AUTOINDEX", "metric_type": "COSINE"}
                )
                for field in ["userid", "document_id", "filename", "file_path", "upload_time", "file_type"]:
                    collection.create_index(
                        field_name=field,
                        index_params={"index_type": "INVERTED"}
                    )
                collection.load()
                logger.debug(f"成功创建并加载集合: {collection_name}")
                return {
                    "status": "success",
                    "collection_name": collection_name,
                    "message": f"集合 {collection_name} 创建成功"
                }
            except Exception as e:
                logger.error(f"创建集合 {collection_name} 失败: {str(e)}")
                raise RuntimeError(f"创建集合失败: {str(e)}")

        except Exception as e:
            logger.error(f"创建集合失败: {str(e)}")
            return {
                "status": "error",
                "collection_name": collection_name,
                "message": str(e)
            }
        finally:
            self.cleanup_milvus_connection()

collection_register('Milvus', MilvusCollection)