import os
import logging
import yaml
from pymilvus import connections, utility, Collection, CollectionSchema, FieldSchema, DataType
from threading import Lock
from llmengine.base_connection import connection_register
from typing import Dict, List
import aiohttp
from langchain_core.documents import Document
from langchain_text_splitters import RecursiveCharacterTextSplitter
import uuid
from datetime import datetime
from filetxt.loader import fileloader
from llmengine.kgc import KnowledgeGraph
import numpy as np
from py2neo import Graph
from scipy.spatial.distance import cosine

logger = logging.getLogger(__name__)

CONFIG_PATH = os.getenv('CONFIG_PATH', '/share/wangmeihua/rag/conf/milvusconfig.yaml')
try:
    with open(CONFIG_PATH, 'r', encoding='utf-8') as f:
        config = yaml.safe_load(f)
    MILVUS_DB_PATH = config['database']['milvus_db_path']
    NEO4J_URI = "bolt://10.18.34.18:7687"
    NEO4J_USER = "neo4j"
    NEO4J_PASSWORD = "261229..wmh"
except Exception as e:
    logger.error(f"加载配置文件 {CONFIG_PATH} 失败: {str(e)}")
    raise RuntimeError(f"无法加载配置文件: {str(e)}")

# 嵌入缓存
EMBED_CACHE = {}

class MilvusConnection:
    _instance = None
    _lock = Lock()

    def __new__(cls):
        with cls._lock:
            if cls._instance is None:
                cls._instance = super(MilvusConnection, cls).__new__(cls)
                cls._instance._initialized = False
            return cls._instance

    def __init__(self):
        if self._initialized:
            return
        self.db_path = MILVUS_DB_PATH
        self.neo4j_uri = NEO4J_URI
        self.neo4j_user = NEO4J_USER
        self.neo4j_password = NEO4J_PASSWORD
        self._initialize_connection()
        self._initialized = True
        logger.info(f"MilvusConnection initialized with db_path: {self.db_path}")

    def _initialize_connection(self):
        """初始化 Milvus 连接，确保单一连接"""
        try:
            db_dir = os.path.dirname(self.db_path)
            if not os.path.exists(db_dir):
                os.makedirs(db_dir, exist_ok=True)
                logger.debug(f"创建 Milvus 目录: {db_dir}")
            if not os.access(db_dir, os.W_OK):
                raise RuntimeError(f"Milvus 目录 {db_dir} 不可写")
            if not connections.has_connection("default"):
                connections.connect("default", uri=self.db_path)
                logger.debug(f"已连接到 Milvus Lite，路径: {self.db_path}")
            else:
                logger.debug("已存在 Milvus 连接，跳过重复连接")
        except Exception as e:
            logger.error(f"连接 Milvus 失败: {str(e)}")
            raise RuntimeError(f"连接 Milvus 失败: {str(e)}")

    async def handle_connection(self, action: str, params: Dict = None) -> Dict:
        """处理数据库操作"""
        try:
            if action == "initialize":
                if not connections.has_connection("default"):
                    self._initialize_connection()
                return {"status": "success", "message": f"Milvus 连接已初始化，路径: {self.db_path}"}
            elif action == "get_params":
                return {"status": "success", "params": {"uri": self.db_path}}
            elif action == "create_collection":
                if not params or "db_type" not in params:
                    return {"status": "error", "message": "缺少 db_type 参数"}
                return self._create_collection(params["db_type"])
            elif action == "delete_collection":
                if not params or "db_type" not in params:
                    return {"status": "error", "message": "缺少 db_type 参数"}
                return self._delete_collection(params["db_type"])
            elif action == "insert_document":
                if not params or "file_path" not in params or "userid" not in params or "db_type" not in params:
                    return {"status": "error", "message": "缺少 file_path, userid 或 db_type 参数"}
                return await self._insert_document(
                    params["file_path"],
                    params["userid"],
                    params["db_type"]
                )
            elif action == "delete_document":
                if not params or "db_type" not in params or "userid" not in params or "filename" not in params:
                    return {"status": "error", "message": "缺少 db_type, userid 或 filename 参数"}
                return self._delete_document(
                    params["db_type"],
                    params["userid"],
                    params["filename"]
                )
            elif action == "fused_search":
                if not params or "query" not in params or "userid" not in params or "db_type" not in params or "file_paths" not in params:
                    return {"status": "error", "message": "缺少 query, userid, db_type 或 file_paths 参数"}
                return await self._fused_search(
                    params["query"],
                    params["userid"],
                    params["db_type"],
                    params["file_paths"],
                    params.get("limit", 5),
                    params.get("offset", 0),
                    params.get("use_rerank", True)
                )
            elif action == "search_query":
                if not params or "query" not in params or "userid" not in params or "db_type" not in params or "file_paths" not in params:
                    return {"status": "error", "message": "缺少 query, userid, db_type 或 file_paths 参数"}
                return await self._search_query(
                    params["query"],
                    params["userid"],
                    params["db_type"],
                    params["file_paths"],
                    params.get("limit", 5),
                    params.get("offset", 0),
                    params.get("use_rerank", True)
                )
            elif action == "list_user_files":
                if not params or "userid" not in params:
                    return {"status": "error", "message": "缺少 userid 参数"}
                return await self.list_user_files(params["userid"])
            else:
                return {"status": "error", "message": f"未知的 action: {action}"}
        except Exception as e:
            logger.error(f"处理操作失败: action={action}, 错误: {str(e)}")
            return {"status": "error", "message": str(e)}

    def _create_collection(self, db_type: str) -> Dict:
        """创建 Milvus 集合"""
        try:
            if not db_type:
                raise ValueError("db_type 不能为空")
            if "_" in db_type:
                raise ValueError("db_type 不能包含下划线")
            if len(db_type) > 100:
                raise ValueError("db_type 的长度应小于 100")

            collection_name = f"ragdb_{db_type}"
            if len(collection_name) > 255:
                raise ValueError(f"集合名称 {collection_name} 超过 255 个字符")
            logger.debug(f"集合名称: {collection_name}")

            fields = [
                FieldSchema(name="pk", dtype=DataType.VARCHAR, is_primary=True, max_length=36, auto_id=True),
                FieldSchema(name="userid", dtype=DataType.VARCHAR, max_length=100),
                FieldSchema(name="document_id", dtype=DataType.VARCHAR, max_length=36),
                FieldSchema(name="text", dtype=DataType.VARCHAR, max_length=65535),
                FieldSchema(name="vector", dtype=DataType.FLOAT_VECTOR, dim=1024),
                FieldSchema(name="filename", dtype=DataType.VARCHAR, max_length=255),
                FieldSchema(name="file_path", dtype=DataType.VARCHAR, max_length=1024),
                FieldSchema(name="upload_time", dtype=DataType.VARCHAR, max_length=64),
                FieldSchema(name="file_type", dtype=DataType.VARCHAR, max_length=64),
            ]
            schema = CollectionSchema(
                fields=fields,
                description=f"{db_type} 数据集合，跨用户使用，包含 document_id 和元数据字段",
                auto_id=True,
                primary_field="pk",
            )

            if utility.has_collection(collection_name):
                try:
                    collection = Collection(collection_name)
                    existing_schema = collection.schema
                    expected_fields = {f.name for f in fields}
                    actual_fields = {f.name for f in existing_schema.fields}
                    vector_field = next((f for f in existing_schema.fields if f.name == "vector"), None)

                    schema_compatible = False
                    if expected_fields == actual_fields and vector_field is not None and vector_field.dtype == DataType.FLOAT_VECTOR:
                        dim = vector_field.params.get('dim', None) if hasattr(vector_field, 'params') and vector_field.params else None
                        schema_compatible = dim == 1024
                        logger.debug(f"检查集合 {collection_name} 的 schema: 字段匹配={expected_fields == actual_fields}, "
                                    f"vector_field存在={vector_field is not None}, dtype={vector_field.dtype if vector_field else '无'}, "
                                    f"dim={dim if dim is not None else '未定义'}")
                    if not schema_compatible:
                        logger.warning(f"集合 {collection_name} 的 schema 不兼容，原因: "
                                      f"字段不匹配: {expected_fields.symmetric_difference(actual_fields) or '无'}, "
                                      f"vector_field: {vector_field is not None}, "
                                      f"dtype: {vector_field.dtype if vector_field else '无'}, "
                                      f"dim: {vector_field.params.get('dim', '未定义') if vector_field and hasattr(vector_field, 'params') and vector_field.params else '未定义'}")
                        utility.drop_collection(collection_name)
                    else:
                        collection.load()
                        logger.debug(f"集合 {collection_name} 已存在并加载成功")
                        return {
                            "status": "success",
                            "collection_name": collection_name,
                            "message": f"集合 {collection_name} 已存在"
                        }
                except Exception as e:
                    logger.error(f"加载集合 {collection_name} 失败: {str(e)}")
                    return {
                        "status": "error",
                        "collection_name": collection_name,
                        "message": str(e)
                    }

            try:
                collection = Collection(collection_name, schema)
                collection.create_index(
                    field_name="vector",
                    index_params={"index_type": "AUTOINDEX", "metric_type": "COSINE"}
                )
                for field in ["userid", "document_id", "filename", "file_path", "upload_time", "file_type"]:
                    collection.create_index(
                        field_name=field,
                        index_params={"index_type": "INVERTED"}
                    )
                collection.load()
                logger.debug(f"成功创建并加载集合: {collection_name}")
                return {
                    "status": "success",
                    "collection_name": collection_name,
                    "message": f"集合 {collection_name} 创建成功"
                }
            except Exception as e:
                logger.error(f"创建集合 {collection_name} 失败: {str(e)}")
                return {
                    "status": "error",
                    "collection_name": collection_name,
                    "message": str(e)
                }
        except Exception as e:
            logger.error(f"创建集合失败: {str(e)}")
            return {
                "status": "error",
                "collection_name": collection_name,
                "message": str(e)
            }

    def _delete_collection(self, db_type: str) -> Dict:
        """删除 Milvus 集合"""
        try:
            if not db_type:
                raise ValueError("db_type 不能为空")
            if "_" in db_type:
                raise ValueError("db_type 不能包含下划线")
            if len(db_type) > 100:
                raise ValueError("db_type 的长度应小于 100")

            collection_name = f"ragdb_{db_type}"
            if len(collection_name) > 255:
                raise ValueError(f"集合名称 {collection_name} 超过 255 个字符")
            logger.debug(f"集合名称: {collection_name}")

            if not utility.has_collection(collection_name):
                logger.debug(f"集合 {collection_name} 不存在")
                return {
                    "status": "success",
                    "collection_name": collection_name,
                    "message": f"集合 {collection_name} 不存在，无需删除"
                }

            try:
                utility.drop_collection(collection_name)
                logger.debug(f"成功删除集合: {collection_name}")
                return {
                    "status": "success",
                    "collection_name": collection_name,
                    "message": f"集合 {collection_name} 删除成功"
                }
            except Exception as e:
                logger.error(f"删除集合 {collection_name} 失败: {str(e)}")
                return {
                    "status": "error",
                    "collection_name": collection_name,
                    "message": str(e)
                }
        except Exception as e:
            logger.error(f"删除集合失败: {str(e)}")
            return {
                "status": "error",
                "collection_name": collection_name,
                "message": str(e)
            }

    async def _insert_document(self, file_path: str, userid: str, db_type: str) -> Dict:
        """将文档插入 Milvus 并抽取三元组到 Neo4j"""
        document_id = str(uuid.uuid4())
        collection_name = f"ragdb_{db_type}"
        try:
            if not userid or not db_type:
                raise ValueError("userid 和 db_type 不能为空")
            if "_" in userid or "_" in db_type:
                raise ValueError("userid 和 db_type 不能包含下划线")
            if not os.path.exists(file_path):
                raise ValueError(f"文件 {file_path} 不存在")
            if len(db_type) > 100:
                raise ValueError("db_type 的长度应小于 100")

            supported_formats = {'pdf', 'doc', 'docx', 'xlsx', 'xls', 'ppt', 'pptx', 'csv', 'txt'}
            ext = file_path.rsplit('.', 1)[1].lower() if '.' in file_path else ''
            if ext not in supported_formats:
                raise ValueError(f"不支持的文件格式: {ext}, 支持的格式: {', '.join(supported_formats)}")

            logger.info(f"生成 document_id: {document_id} for file: {file_path}")

            logger.debug(f"加载文件: {file_path}")
            text = fileloader(file_path)
            if not text or not text.strip():
                raise ValueError(f"文件 {file_path} 加载为空")

            document = Document(page_content=text)
            text_splitter = RecursiveCharacterTextSplitter(
                chunk_size=2000,
                chunk_overlap=200,
                length_function=len,
            )
            logger.debug("开始分片文件内容")
            chunks = text_splitter.split_documents([document])
            if not chunks:
                raise ValueError(f"文件 {file_path} 未生成任何文档块")
            logger.debug(f"文件 {file_path} 分割为 {len(chunks)} 个文档块")

            filename = os.path.basename(file_path).rsplit('.', 1)[0]
            upload_time = datetime.now().isoformat()
            documents = []
            for i, chunk in enumerate(chunks):
                chunk.metadata.update({
                    'userid': userid,
                    'document_id': document_id,
                    'filename': filename + '.' + ext,
                    'file_path': file_path,
                    'upload_time': upload_time,
                    'file_type': ext,
                })
                documents.append(chunk)
                logger.debug(f"文档块 {i} 元数据: {chunk.metadata}")

            logger.debug(f"确保集合 {collection_name} 存在")
            create_result = self._create_collection(db_type)
            if create_result["status"] == "error":
                raise RuntimeError(f"集合创建失败: {create_result['message']}")

            logger.debug("调用嵌入服务生成向量")
            texts = [doc.page_content for doc in documents]
            embeddings = await self._get_embeddings(texts)
            await self._insert_to_milvus(collection_name, documents, embeddings)
            logger.info(f"成功插入 {len(documents)} 个文档块到 {collection_name}")

            logger.debug("调用三元组抽取服务")
            try:
                triples = await self._extract_triples(text)
                if triples:
                    logger.debug(f"抽取到 {len(triples)} 个三元组，插入 Neo4j")
                    kg = KnowledgeGraph(triples=triples, document_id=document_id)
                    kg.create_graphnodes()
                    kg.create_graphrels()
                    kg.export_data()
                    logger.info(f"文件 {file_path} 三元组成功插入 Neo4j")
                else:
                    logger.warning(f"文件 {file_path} 未抽取到三元组")
            except Exception as e:
                logger.warning(f"处理三元组失败: {str(e)}, 但不影响 Milvus 插入")

            return {
                "status": "success",
                "document_id": document_id,
                "collection_name": collection_name,
                "message": f"文件 {file_path} 成功嵌入并处理三元组"
            }

        except Exception as e:
            logger.error(f"插入文档失败: {str(e)}")
            return {
                "status": "error",
                "document_id": document_id,
                "collection_name": collection_name,
                "message": str(e)
            }

    async def _get_embeddings(self, texts: List[str]) -> List[List[float]]:
        """调用嵌入服务获取文本的向量，带缓存"""
        try:
            # 检查缓存
            uncached_texts = [text for text in texts if text not in EMBED_CACHE]
            if uncached_texts:
                async with aiohttp.ClientSession() as session:
                    async with session.post(
                        "http://localhost:9998/v1/embeddings",
                        headers={"Content-Type": "application/json"},
                        json={"input": uncached_texts}
                    ) as response:
                        if response.status != 200:
                            logger.error(f"嵌入服务调用失败，状态码: {response.status}")
                            raise RuntimeError(f"嵌入服务调用失败: {response.status}")
                        result = await response.json()
                        if result.get("object") != "list" or not result.get("data"):
                            logger.error(f"嵌入服务响应格式错误: {result}")
                            raise RuntimeError("嵌入服务响应格式错误")
                        embeddings = [item["embedding"] for item in result["data"]]
                        for text, embedding in zip(uncached_texts, embeddings):
                            EMBED_CACHE[text] = np.array(embedding) / np.linalg.norm(embedding)
                        logger.debug(f"成功获取 {len(embeddings)} 个新嵌入向量，缓存大小: {len(EMBED_CACHE)}")
            # 返回缓存中的嵌入
            return [EMBED_CACHE[text] for text in texts]
        except Exception as e:
            logger.error(f"嵌入服务调用失败: {str(e)}")
            raise RuntimeError(f"嵌入服务调用失败: {str(e)}")

    async def _extract_triples(self, text: str) -> List[Dict]:
        """调用三元组抽取服务"""
        try:
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    "http://localhost:9991/v1/triples",
                    headers={"Content-Type": "application/json; charset=utf-8"},
                    json={"text": text}
                ) as response:
                    if response.status != 200:
                        logger.error(f"三元组抽取服务调用失败，状态码: {response.status}")
                        raise RuntimeError(f"三元组抽取服务调用失败: {response.status}")
                    result = await response.json()
                    if result.get("object") != "list" or not result.get("data"):
                        logger.error(f"三元组抽取服务响应格式错误: {result}")
                        raise RuntimeError("三元组抽取服务响应格式错误")
                    triples = result["data"]
                    logger.debug(f"成功抽取 {len(triples)} 个三元组")
                    return triples
        except Exception as e:
            logger.error(f"三元组抽取服务调用失败: {str(e)}")
            raise RuntimeError(f"三元组抽取服务调用失败: {str(e)}")

    async def _insert_to_milvus(self, collection_name: str, documents: List[Document], embeddings: List[List[float]]) -> None:
        """将文档和嵌入向量插入 Milvus 集合"""
        try:
            if not connections.has_connection("default"):
                self._initialize_connection()
            collection = Collection(collection_name)
            collection.load()
            data = {
                "userid": [doc.metadata["userid"] for doc in documents],
                "document_id": [doc.metadata["document_id"] for doc in documents],
                "text": [doc.page_content for doc in documents],
                "vector": embeddings,
                "filename": [doc.metadata["filename"] for doc in documents],
                "file_path": [doc.metadata["file_path"] for doc in documents],
                "upload_time": [doc.metadata["upload_time"] for doc in documents],
                "file_type": [doc.metadata["file_type"] for doc in documents],
            }
            collection.insert([data[field.name] for field in collection.schema.fields if field.name != "pk"])
            collection.flush()
            logger.debug(f"成功插入 {len(documents)} 个文档到集合 {collection_name}")
        except Exception as e:
            logger.error(f"插入 Milvus 失败: {str(e)}")
            raise RuntimeError(f"插入 Milvus 失败: {str(e)}")

    def _delete_document(self, db_type: str, userid: str, filename: str) -> Dict:
        """删除用户指定文件数据"""
        collection_name = f"ragdb_{db_type}"
        try:
            if not db_type or "_" in db_type:
                raise ValueError("db_type 不能为空且不能包含下划线")
            if not userid or "_" in userid:
                raise ValueError("userid 不能为空且不能包含下划线")
            if not filename:
                raise ValueError("filename 不能为空")
            if len(db_type) > 100 or len(userid) > 100 or len(filename) > 255:
                raise ValueError("db_type、userid 或 filename 的长度超出限制")

            if not utility.has_collection(collection_name):
                logger.warning(f"集合 {collection_name} 不存在")
                return {
                    "status": "error",
                    "collection_name": collection_name,
                    "message": f"集合 {collection_name} 不存在"
                }

            try:
                collection = Collection(collection_name)
                collection.load()
                logger.debug(f"加载集合: {collection_name}")
            except Exception as e:
                logger.error(f"加载集合 {collection_name} 失败: {str(e)}")
                return {
                    "status": "error",
                    "collection_name": collection_name,
                    "message": f"加载集合失败: {str(e)}"
                }

            expr = f"userid == '{userid}' and filename == '{filename}'"
            logger.debug(f"查询表达式: {expr}")
            try:
                results = collection.query(
                    expr=expr,
                    output_fields=["document_id"],
                    limit=1000
                )
                if not results:
                    logger.warning(f"没有找到 userid={userid}, filename={filename} 的记录")
                    return {
                        "status": "error",
                        "collection_name": collection_name,
                        "message": f"没有找到 userid={userid}, filename={filename} 的记录"
                    }
                document_ids = list(set(result["document_id"] for result in results if "document_id" in result))
                logger.debug(f"找到 {len(document_ids)} 个 document_id: {document_ids}")
            except Exception as e:
                logger.error(f"查询 document_id 失败: {str(e)}")
                return {
                    "status": "error",
                    "collection_name": collection_name,
                    "message": f"查询失败: {str(e)}"
                }

            total_deleted = 0
            for doc_id in document_ids:
                try:
                    delete_expr = f"userid == '{userid}' and document_id == '{doc_id}'"
                    logger.debug(f"删除表达式: {delete_expr}")
                    delete_result = collection.delete(delete_expr)
                    deleted_count = delete_result.delete_count
                    total_deleted += deleted_count
                    logger.info(f"成功删除 document_id={doc_id} 的 {deleted_count} 条记录")
                except Exception as e:
                    logger.error(f"删除 document_id={doc_id} 失败: {str(e)}")
                    continue

            if total_deleted == 0:
                logger.warning(f"没有删除任何记录，userid={userid}, filename={filename}")
                return {
                    "status": "error",
                    "collection_name": collection_name,
                    "message": f"没有删除任何记录，userid={userid}, filename={filename}"
                }

            logger.info(f"总计删除 {total_deleted} 条记录，userid={userid}, filename={filename}")
            return {
                "status": "success",
                "collection_name": collection_name,
                "message": f"成功删除 {total_deleted} 条记录，userid={userid}, filename={filename}"
            }

        except Exception as e:
            logger.error(f"删除文档失败: {str(e)}")
            return {
                "status": "error",
                "collection_name": collection_name,
                "message": f"删除文档失败: {str(e)}"
            }

    async def _extract_entities(self, query: str) -> List[str]:
        """调用实体识别服务"""
        try:
            if not query:
                raise ValueError("查询文本不能为空")
            async with aiohttp.ClientSession() as session:
                async with session.post(
                    "http://localhost:9990/v1/entities",
                    headers={"Content-Type": "application/json"},
                    json={"query": query}
                ) as response:
                    if response.status != 200:
                        logger.error(f"实体识别服务调用失败，状态码: {response.status}")
                        raise RuntimeError(f"实体识别服务调用失败: {response.status}")
                    result = await response.json()
                    if result.get("object") != "list" or not result.get("data"):
                        logger.error(f"实体识别服务响应格式错误: {result}")
                        raise RuntimeError("实体识别服务响应格式错误")
                    entities = result["data"]
                    unique_entities = list(dict.fromkeys(entities))  # 去重
                    logger.debug(f"成功提取 {len(unique_entities)} 个唯一实体: {unique_entities}")
                    return unique_entities
        except Exception as e:
            logger.error(f"实体识别服务调用失败: {str(e)}")
            return []

    async def _match_triplets(self, query: str, query_entities: List[str], userid: str, document_id: str) -> List[Dict]:
        """匹配查询实体与 Neo4j 中的三元组"""
        matched_triplets = []
        ENTITY_SIMILARITY_THRESHOLD = 0.8

        try:
            graph = Graph(self.neo4j_uri, auth=(self.neo4j_user, self.neo4j_password))
            logger.debug(f"已连接到 Neo4j: {self.neo4j_uri}")

            matched_names = set()
            for entity in query_entities:
                normalized_entity = entity.lower().strip()
                query = """
                MATCH (n {document_id: $document_id})
                WHERE toLower(n.name) CONTAINS $entity
                   OR apoc.text.levenshteinSimilarity(toLower(n.name), $entity) > 0.7
                RETURN n.name, apoc.text.levenshteinSimilarity(toLower(n.name), $entity) AS sim
                ORDER BY sim DESC
                LIMIT 100
                """
                try:
                    results = graph.run(query, document_id=document_id, entity=normalized_entity).data()
                    for record in results:
                        matched_names.add(record['n.name'])
                        logger.debug(f"实体 {entity} 匹配节点: {record['n.name']} (Levenshtein 相似度: {record['sim']:.2f})")
                except Exception as e:
                    logger.warning(f"模糊匹配实体 {entity} 失败: {str(e)}")
                    continue

            triplets = []
            if matched_names:
                query = """
                MATCH (h {document_id: $document_id})-[r]->(t {document_id: $document_id})
                WHERE h.name IN $matched_names OR t.name IN $matched_names
                RETURN h.name AS head, r.name AS type, t.name AS tail
                LIMIT 100
                """
                try:
                    results = graph.run(query, document_id=document_id, matched_names=list(matched_names)).data()
                    seen = set()
                    for record in results:
                        head, type_, tail = record['head'], record['type'], record['tail']
                        triplet_key = (head.lower(), type_.lower(), tail.lower())
                        if triplet_key not in seen:
                            seen.add(triplet_key)
                            triplets.append({
                                'head': head,
                                'type': type_,
                                'tail': tail,
                                'head_type': '',
                                'tail_type': ''
                            })
                    logger.debug(f"从 Neo4j 加载三元组: document_id={document_id}, 数量={len(triplets)}")
                except Exception as e:
                    logger.error(f"检索三元组失败: document_id={document_id}, 错误: {str(e)}")
                    return []

            if not triplets:
                logger.debug(f"文档 document_id={document_id} 无匹配三元组")
                return []

            texts_to_embed = query_entities + [t['head'] for t in triplets] + [t['tail'] for t in triplets]
            embeddings = await self._get_embeddings(texts_to_embed)
            entity_vectors = {entity: embeddings[i] for i, entity in enumerate(query_entities)}
            head_vectors = {t['head']: embeddings[len(query_entities) + i] for i, t in enumerate(triplets)}
            tail_vectors = {t['tail']: embeddings[len(query_entities) + len(triplets) + i] for i, t in enumerate(triplets)}
            logger.debug(f"成功获取 {len(embeddings)} 个嵌入向量（{len(query_entities)} entities + {len(triplets)} heads + {len(triplets)} tails）")

            for entity in query_entities:
                entity_vec = entity_vectors[entity]
                for d_triplet in triplets:
                    d_head_vec = head_vectors[d_triplet['head']]
                    d_tail_vec = tail_vectors[d_triplet['tail']]
                    head_similarity = 1 - cosine(entity_vec, d_head_vec)
                    tail_similarity = 1 - cosine(entity_vec, d_tail_vec)

                    if head_similarity >= ENTITY_SIMILARITY_THRESHOLD or tail_similarity >= ENTITY_SIMILARITY_THRESHOLD:
                        matched_triplets.append(d_triplet)
                        logger.debug(f"匹配三元组: {d_triplet['head']} - {d_triplet['type']} - {d_triplet['tail']} "
                                     f"(entity={entity}, head_sim={head_similarity:.2f}, tail_sim={tail_similarity:.2f})")

            unique_matched = []
            seen = set()
            for t in matched_triplets:
                identifier = (t['head'].lower(), t['type'].lower(), t['tail'].lower())
                if identifier not in seen:
                    seen.add(identifier)
                    unique_matched.append(t)

            logger.info(f"找到 {len(unique_matched)} 个匹配的三元组")
            return unique_matched

        except Exception as e:
            logger.error(f"匹配三元组失败: {str(e)}")
            return []

            texts_to_embed = query_entities + [t['head'] for t in triplets] + [t['tail'] for t in triplets]
            embeddings = await self._get_embeddings(texts_to_embed)
            entity_vectors = {entity: embeddings[i] for i, entity in enumerate(query_entities)}
            head_vectors = {t['head']: embeddings[len(query_entities) + i] for i, t in enumerate(triplets)}
            tail_vectors = {t['tail']: embeddings[len(query_entities) + len(triplets) + i] for i, t in enumerate(triplets)}
            logger.debug(f"成功获取 {len(embeddings)} 个嵌入向量（{len(query_entities)} entities + {len(triplets)} heads + {len(triplets)} tails）")

            for entity in query_entities:
                entity_vec = entity_vectors[entity]
                for d_triplet in triplets:
                    d_head_vec = head_vectors[d_triplet['head']]
                    d_tail_vec = tail_vectors[d_triplet['tail']]
                    head_similarity = 1 - cosine(entity_vec, d_head_vec)
                    tail_similarity = 1 - cosine(entity_vec, d_tail_vec)

                    if head_similarity >= ENTITY_SIMILARITY_THRESHOLD or tail_similarity >= ENTITY_SIMILARITY_THRESHOLD:
                        matched_triplets.append(d_triplet)
                        logger.debug(f"匹配三元组: {d_triplet['head']} - {d_triplet['type']} - {d_triplet['tail']} "
                                     f"(entity={entity}, head_sim={head_similarity:.2f}, tail_sim={tail_similarity:.2f})")

            unique_matched = []
            seen = set()
            for t in matched_triplets:
                identifier = (t['head'].lower(), t['type'].lower(), t['tail'].lower())
                if identifier not in seen:
                    seen.add(identifier)
                    unique_matched.append(t)

            logger.info(f"找到 {len(unique_matched)} 个匹配的三元组")
            return unique_matched

        except Exception as e:
            logger.error(f"匹配三元组失败: {str(e)}")
            return []

    async def _rerank_results(self, query: str, results: List[Dict], top_n: int) -> List[Dict]:
        """调用重排序服务

        Args:
            query (str): 查询字符串
            results (List[Dict]): 原始搜索结果列表
            top_n (int): 返回的重排序结果数量

        Returns:
            List[Dict]: 重排序后的结果列表
        """
        try:
            if not results:
                logger.debug("无结果需要重排序")
                return results

            # 验证 top_n
            if not isinstance(top_n, int) or top_n < 1:
                logger.warning(f"无效的 top_n 参数: {top_n}，使用 len(results)={len(results)}")
                top_n = len(results)
            else:
                top_n = min(top_n, len(results))  # 确保不超过结果数量
            logger.debug(f"重排序 top_n={top_n}, 原始结果数={len(results)}")

            documents = [result["text"] for result in results]
            async with aiohttp.ClientSession() as session:
                async with session.post(
                        "http://localhost:9997/v1/rerank",
                        headers={"Content-Type": "application/json"},
                        json={
                            "model": "rerank-001",
                            "query": query,
                            "documents": documents,
                            "top_n": top_n
                        }
                ) as response:
                    if response.status != 200:
                        logger.error(f"重排序服务调用失败，状态码: {response.status}")
                        raise RuntimeError(f"重排序服务调用失败: {response.status}")
                    result = await response.json()
                    if result.get("object") != "rerank.result" or not result.get("data"):
                        logger.error(f"重排序服务响应格式错误: {result}")
                        raise RuntimeError("重排序服务响应格式错误")
                    rerank_data = result["data"]
                    reranked_results = []
                    for item in rerank_data:
                        index = item["index"]
                        if index < len(results):
                            results[index]["rerank_score"] = item["relevance_score"]
                            reranked_results.append(results[index])
                    logger.debug(f"成功重排序 {len(reranked_results)} 条结果")
                    return reranked_results[:top_n]  # 确保返回不超过 top_n 条
        except Exception as e:
            logger.error(f"重排序服务调用失败: {str(e)}")
            return results  # 出错时返回原始结果

    async def _fused_search(self, query: str, userid: str, db_type: str, file_paths: List[str], limit: int = 5,
                            offset: int = 0, use_rerank: bool = True) -> List[Dict]:
        """融合搜索，将查询与所有三元组拼接后向量化搜索"""
        try:
            logger.info(
                f"开始融合搜索: query={query}, userid={userid}, db_type={db_type}, file_paths={file_paths}, limit={limit}, offset={offset}, use_rerank={use_rerank}")
            collection_name = f"ragdb_{db_type}"

            # 参数验证
            if not query or not userid or not db_type or not file_paths:
                raise ValueError("query、userid、db_type 和 file_paths 不能为空")
            if "_" in userid or "_" in db_type:
                raise ValueError("userid 和 db_type 不能包含下划线")
            if len(db_type) > 100 or len(userid) > 100:
                raise ValueError("db_type 或 userid 的长度超出限制")
            if limit < 1 or offset < 0:
                raise ValueError("limit 必须大于 0，offset 必须大于或等于 0")

            # 检查集合是否存在
            if not utility.has_collection(collection_name):
                logger.warning(f"集合 {collection_name} 不存在")
                return []

            # 加载集合
            try:
                collection = Collection(collection_name)
                collection.load()
                logger.debug(f"加载集合: {collection_name}")
            except Exception as e:
                logger.error(f"加载集合 {collection_name} 失败: {str(e)}")
                return []

            # 提取实体
            query_entities = await self._extract_entities(query)
            logger.debug(f"提取实体: {query_entities}")

            # 收集 document_id 和三元组
            documents = []
            all_triplets = []
            for file_path in file_paths:
                filename = os.path.basename(file_path)
                logger.debug(f"处理文件: {filename}")

                # 获取 document_id
                results = collection.query(
                    expr=f"userid == '{userid}' and filename == '{filename}'",
                    output_fields=["document_id", "filename"],
                    limit=1
                )
                if not results:
                    logger.warning(f"未找到 userid {userid} 和 filename {filename} 对应的文档")
                    continue
                documents.append(results[0])

                # 获取三元组
                document_id = results[0]["document_id"]
                matched_triplets = await self._match_triplets(query, query_entities, userid, document_id)
                logger.debug(f"文件 {filename} 匹配三元组: {len(matched_triplets)} 条")
                all_triplets.extend(matched_triplets)

            if not documents:
                logger.warning("未找到任何有效文档")
                return []

            logger.info(f"找到 {len(documents)} 个文档: {[doc['filename'] for doc in documents]}")

            # 拼接查询和三元组
            triplet_texts = []
            for triplet in all_triplets:
                head = triplet.get('head', '')
                type_ = triplet.get('type', '')
                tail = triplet.get('tail', '')
                if head and type_ and tail:
                    triplet_texts.append(f"{head} {type_} {tail}")
                else:
                    logger.debug(f"无效三元组: {triplet}")
            combined_text = query
            if triplet_texts:
                combined_text += " [三元组] " + "; ".join(triplet_texts)
            logger.debug(
                f"拼接文本: {combined_text[:200]}... (总长度: {len(combined_text)}, 三元组数量: {len(triplet_texts)})")

            # 生成拼接文本的嵌入向量
            embeddings = await self._get_embeddings([combined_text])
            query_vector = embeddings[0]
            logger.debug(f"拼接文本向量维度: {len(query_vector)}")

            # 构造搜索参数
            search_params = {"metric_type": "COSINE", "params": {"nprobe": 10}}

            # 构造过滤表达式
            filenames = [os.path.basename(file_path) for file_path in file_paths]
            filename_expr = " or ".join([f"filename == '{filename}'" for filename in filenames])
            expr = f"userid == '{userid}' and ({filename_expr})"
            logger.debug(f"搜索表达式: {expr}")

            # 执行向量搜索
            try:
                results = collection.search(
                    data=[query_vector],
                    anns_field="vector",
                    param=search_params,
                    limit=limit,
                    expr=expr,
                    output_fields=["text", "userid", "document_id", "filename", "file_path", "upload_time",
                                   "file_type"],
                    offset=offset
                )
            except Exception as e:
                logger.error(f"向量搜索失败: {str(e)}")
                return []

            # 处理搜索结果
            search_results = []
            for hits in results:
                for hit in hits:
                    metadata = {
                        "userid": hit.entity.get("userid"),
                        "document_id": hit.entity.get("document_id"),
                        "filename": hit.entity.get("filename"),
                        "file_path": hit.entity.get("file_path"),
                        "upload_time": hit.entity.get("upload_time"),
                        "file_type": hit.entity.get("file_type")
                    }
                    result = {
                        "text": hit.entity.get("text"),
                        "distance": hit.distance,
                        "source": "fused_query_with_triplets",
                        "metadata": metadata
                    }
                    search_results.append(result)
                    logger.debug(
                        f"搜索命中: text={result['text'][:100]}..., distance={hit.distance}, source={result['source']}")

            # 去重
            unique_results = []
            seen_texts = set()
            for result in sorted(search_results, key=lambda x: x['distance'], reverse=True):
                if result['text'] not in seen_texts:
                    unique_results.append(result)
                    seen_texts.add(result['text'])
            logger.info(f"去重后结果数量: {len(unique_results)} (原始数量: {len(search_results)})")

            # 重排序（可选）
            if use_rerank and unique_results:
                logger.debug("开始重排序")
                unique_results = await self._rerank_results(combined_text, unique_results, 5)
                unique_results = sorted(unique_results, key=lambda x: x.get('rerank_score', 0), reverse=True)
                logger.debug(f"重排序分数分布: {[round(r.get('rerank_score', 0), 3) for r in unique_results]}")
            else:
                unique_results = [{k: v for k, v in r.items() if k != 'rerank_score'} for r in unique_results]

            logger.info(f"融合搜索完成，返回 {len(unique_results)} 条结果")
            return unique_results[:limit]

        except Exception as e:
            logger.error(f"融合搜索失败: {str(e)}")
            return []

    async def _search_query(self, query: str, userid: str, db_type: str, file_paths: List[str], limit: int = 5,
                            offset: int = 0, use_rerank: bool = True) -> List[Dict]:
        """纯向量搜索，基于查询文本在指定文档中搜索相关文本块"""
        try:
            logger.info(
                f"开始纯向量搜索: query={query}, userid={userid}, db_type={db_type}, file_paths={file_paths}, limit={limit}, offset={offset}, use_rerank={use_rerank}")
            collection_name = f"ragdb_{db_type}"

            # 参数验证
            if not query:
                raise ValueError("查询文本不能为空")
            if not userid or not db_type:
                raise ValueError("userid 和 db_type 不能为空")
            if "_" in userid or "_" in db_type:
                raise ValueError("userid 和 db_type 不能包含下划线")
            if len(userid) > 100 or len(db_type) > 100:
                raise ValueError("userid 或 db_type 的长度超出限制")
            if limit <= 0 or limit > 16384:
                raise ValueError("limit 必须在 1 到 16384 之间")
            if offset < 0:
                raise ValueError("offset 不能为负数")
            if limit + offset > 16384:
                raise ValueError("limit + offset 不能超过 16384")
            if not file_paths:
                raise ValueError("file_paths 不能为空")
            for file_path in file_paths:
                if not isinstance(file_path, str):
                    raise ValueError(f"file_path 必须是字符串: {file_path}")
                if len(os.path.basename(file_path)) > 255:
                    raise ValueError(f"文件名长度超出 255 个字符: {file_path}")
                if "_" in os.path.basename(file_path):
                    raise ValueError(f"文件名 {file_path} 不能包含下划线")

            # 检查集合是否存在
            if not utility.has_collection(collection_name):
                logger.warning(f"集合 {collection_name} 不存在")
                return []

            # 加载集合
            try:
                collection = Collection(collection_name)
                collection.load()
                logger.debug(f"加载集合: {collection_name}")
            except Exception as e:
                logger.error(f"加载集合 {collection_name} 失败: {str(e)}")
                raise RuntimeError(f"加载集合失败: {str(e)}")

            # 生成查询向量
            embeddings = await self._get_embeddings([query])
            query_vector = embeddings[0]
            logger.debug(f"查询向量维度: {len(query_vector)}")

            # 构造搜索参数
            search_params = {"metric_type": "COSINE", "params": {"nprobe": 10}}

            # 构造过滤表达式
            filenames = [os.path.basename(file_path) for file_path in file_paths]
            filename_expr = " or ".join([f"filename == '{filename}'" for filename in filenames])
            expr = f"userid == '{userid}' and ({filename_expr})"
            logger.debug(f"搜索表达式: {expr}")

            # 执行搜索
            try:
                results = collection.search(
                    data=[query_vector],
                    anns_field="vector",
                    param=search_params,
                    limit=limit,
                    expr=expr,
                    output_fields=["text", "userid", "document_id", "filename", "file_path", "upload_time",
                                   "file_type"],
                    offset=offset
                )
            except Exception as e:
                logger.error(f"搜索失败: {str(e)}")
                raise RuntimeError(f"搜索失败: {str(e)}")

            # 处理搜索结果
            search_results = []
            for hits in results:
                for hit in hits:
                    metadata = {
                        "userid": hit.entity.get("userid"),
                        "document_id": hit.entity.get("document_id"),
                        "filename": hit.entity.get("filename"),
                        "file_path": hit.entity.get("file_path"),
                        "upload_time": hit.entity.get("upload_time"),
                        "file_type": hit.entity.get("file_type")
                    }
                    result = {
                        "text": hit.entity.get("text"),
                        "distance": hit.distance,
                        "source": "vector_query",
                        "metadata": metadata
                    }
                    search_results.append(result)
                    logger.debug(
                        f"命中: text={result['text'][:100]}..., distance={hit.distance}, filename={metadata['filename']}")

            # 去重
            unique_results = []
            seen_texts = set()
            for result in sorted(search_results, key=lambda x: x['distance'], reverse=True):
                if result['text'] not in seen_texts:
                    unique_results.append(result)
                    seen_texts.add(result['text'])
            logger.info(f"去重后结果数量: {len(unique_results)} (原始数量: {len(search_results)})")

            # 重排序（可选）
            if use_rerank and unique_results:
                logger.debug("开始重排序")
                unique_results = await self._rerank_results(query, unique_results, 5)
                unique_results = sorted(unique_results, key=lambda x: x.get('rerank_score', 0), reverse=True)
                logger.debug(f"重排序分数分布: {[round(r.get('rerank_score', 0), 3) for r in unique_results]}")
            else:
                # 未启用重排序，确保不包含 rerank_score
                unique_results = [{k: v for k, v in r.items() if k != 'rerank_score'} for r in unique_results]

            logger.info(f"纯向量搜索完成，返回 {len(unique_results)} 条结果")
            return unique_results[:limit]

        except Exception as e:
            logger.error(f"纯向量搜索失败: {str(e)}")
            return []

    async def list_user_files(self, userid: str) -> List[Dict]:
        """根据 userid 返回用户的所有文件列表，从所有 ragdb_ 开头的集合中查询

        Args:
            userid (str): 用户 ID

        Returns:
            List[Dict]: 文件列表，每个文件包含 filename, file_path, db_type, upload_time, file_type
        """
        try:
            logger.info(f"开始查询用户文件列表: userid={userid}")

            # 参数验证
            if not userid:
                raise ValueError("userid 不能为空")
            if "_" in userid:
                raise ValueError("userid 不能包含下划线")
            if len(userid) > 100:
                raise ValueError("userid 长度超出限制")

            # 获取所有 ragdb_ 开头的集合
            collections = [c for c in utility.list_collections() if c.startswith("ragdb_")]
            if not collections:
                logger.warning("未找到任何 ragdb_ 开头的集合")
                return []
            logger.debug(f"找到集合: {collections}")

            # 收集文件信息
            file_list = []
            seen_files = set()  # 用于去重 (filename, file_path)
            for collection_name in collections:
                # 提取 db_type
                db_type = collection_name.replace("ragdb_", "")
                logger.debug(f"处理集合: {collection_name}, db_type={db_type}")

                # 加载集合
                try:
                    collection = Collection(collection_name)
                    collection.load()
                    logger.debug(f"加载集合: {collection_name}")
                except Exception as e:
                    logger.error(f"加载集合 {collection_name} 失败: {str(e)}")
                    continue

                # 查询文本块
                try:
                    results = collection.query(
                        expr=f"userid == '{userid}'",
                        output_fields=["filename", "file_path", "upload_time", "file_type"],
                        limit=1000  # 假设最大返回 1000 个文本块
                    )
                    logger.debug(f"集合 {collection_name} 查询到 {len(results)} 个文本块")
                except Exception as e:
                    logger.error(f"查询集合 {collection_name} 失败: userid={userid}, 错误: {str(e)}")
                    continue

                # 处理查询结果
                for result in results:
                    filename = result.get("filename")
                    file_path = result.get("file_path")
                    upload_time = result.get("upload_time")
                    file_type = result.get("file_type")
                    if (filename, file_path) not in seen_files:
                        seen_files.add((filename, file_path))
                        file_list.append({
                            "filename": filename,
                            "file_path": file_path,
                            "db_type": db_type,
                            "upload_time": upload_time,
                            "file_type": file_type
                        })
                        logger.debug(
                            f"文件: filename={filename}, file_path={file_path}, db_type={db_type}, upload_time={upload_time}, file_type={file_type}")

            logger.info(f"返回 {len(file_list)} 个文件")
            return sorted(file_list, key=lambda x: x["upload_time"], reverse=True)  # 按上传时间降序排序

        except Exception as e:
            logger.error(f"查询用户文件列表失败: userid={userid}, 错误: {str(e)}")
            return []
connection_register('Milvus', MilvusConnection)
logger.info("MilvusConnection registered")