from traceback import format_exc
import os
import argparse
import logging
import yaml
from typing import List
from base_triple import get_llm_class
from mrebeltriple import MRebelTripleExtractor
from appPublic.registerfunction import RegisterFunction
from appPublic.log import debug, exception
from ahserver.serverenv import ServerEnv
from ahserver.globalEnv import stream_response
from ahserver.webapp import webserver
import aiohttp.web

# 配置日志
logger = logging.getLogger('llmengine_triple')
logger.setLevel(logging.DEBUG)
log_file = '/share/wangmeihua/rag/logs/llmengine_triple.log'
os.makedirs(os.path.dirname(log_file), exist_ok=True)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
for handler in (logging.FileHandler(log_file, encoding='utf-8'), logging.StreamHandler()):
    handler.setFormatter(formatter)
    logger.addHandler(handler)

# 加载配置文件
CONFIG_PATH = os.getenv('CONFIG_PATH', '/share/wangmeihua/rag/conf/milvusconfig.yaml')
try:
    with open(CONFIG_PATH, 'r', encoding='utf-8') as f:
        config = yaml.safe_load(f)
except Exception as e:
    logger.error(f"Failed to load config {CONFIG_PATH}: {str(e)}")
    raise RuntimeError(f"Failed to load config: {str(e)}")

helptext = """mREBEL Triplets API:

1. Triplets Endpoint:
path: /v1/triples
headers: {
    "Content-Type": "application/json"
}
data: {
    "text": "知识图谱是一个结构化的语义知识库。"
}
response: {
    "object": "list",
    "data": [
        {
            "head": "知识图谱",
            "head_type": "Concept",
            "type": "is_a",
            "tail": "语义知识库",
            "tail_type": "Concept"
        },
        ...
    ]
}

2. Docs Endpoint:
path: /v1/docs
response: This help text
"""

def init():
    rf = RegisterFunction()
    rf.register('triples', triples)
    rf.register('docs', docs)

async def docs(request, params_kw, *params, **kw):
    return helptext

async def triples(request, params_kw, *params, **kw):
    try:
        debug(f'Received params_kw: {params_kw}')
        # 显式解析请求数据
        if not params_kw:
            try:
                data = await request.json()
                params_kw = data
                debug(f'Parsed JSON data: {params_kw}')
            except Exception as e:
                logger.error(f"Failed to parse JSON: {str(e)}")
                raise aiohttp.web.HTTPBadRequest(reason=f"Invalid JSON: {str(e)}")

        se = ServerEnv()
        engine = se.engine
        if engine is None:
            raise ValueError("Engine not initialized")

        text = params_kw.get('text')
        if not text:
            e = ValueError("text cannot be empty")
            exception(f'{e}')
            raise e

        triplets = await engine.extract_triplets(text)
        debug(f'{triplets=}')
        return {
            "object": "list",
            "data": triplets
        }
    except Exception as e:
        logger.error(f"Error in triples endpoint: {str(e)}")
        logger.debug(f"Traceback: {format_exc()}")
        raise

def main():
    parser = argparse.ArgumentParser(prog="mREBEL Triplet Service")
    parser.add_argument('-w', '--workdir', default=None)
    parser.add_argument('-p', '--port', type=int, default=9991)
    parser.add_argument('model_path')
    args = parser.parse_args()

    try:
        Klass = get_llm_class(args.model_path)
        if Klass is None:
            e = Exception(f"{args.model_path} has no mapping to a model class")
            exception(f'{e}, {format_exc()}')
            raise e

        se = ServerEnv()
        se.engine = Klass(args.model_path)
        workdir = args.workdir or os.getcwd()
        port = args.port
        debug(f'{args=}')
        logger.info(f"Starting mREBEL Triplet Service on port {port}, model: {args.model_path}")
        webserver(init, workdir, port)
    except Exception as e:
        logger.error(f"Failed to start server: {str(e)}")
        logger.debug(f"Traceback: {format_exc()}")
        raise

if __name__ == "__main__":
    main()